package weaver.interfaces.sskj.action;

import cn.hutool.core.date.DateUtil;
import com.engine.kq.biz.KQTimesArrayComInfo;
import com.engine.sskj.util.SskjUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.Util;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.*;

import java.util.*;

/**
 * 加班申请流程转建模数据
 */
@Slf4j
public class WorkOverTimeAction implements Action {

    @Override
    public String execute(RequestInfo requestInfo) {
        BaseBean bb = new BaseBean();
        RecordSet rs = new RecordSet();
        SskjUtil sskjUtil = new SskjUtil();
        KQTimesArrayComInfo kqTimesArrayComInfo = new KQTimesArrayComInfo();
        //获取流程requestId
        String requestId = Util.null2String(requestInfo.getRequestid());
        // 根据requestId查询考勤建模表信息
        List<Map<String, String>> list = getKqFlowOverTimeInfo(requestId);
        //遍历数据更新
        for (Map<String, String> map : list) {
            // 部门
            String bm = map.get("departmentid");
            // 姓名
            String xm = map.get("resourceid");
            //日期
            String fromdate = map.get("fromdate").substring(0, 7);
            // 加班类型
            String changetype = map.get("changetype");
            // 加班类别
            String field28 = map.get("field28");
            // 获取建模主表名称
            String mainTableName = getMainTableName(field28);
            // 1.先判断建模主表是否有记录
            String id = "";
            String sql2 = " select id from " + mainTableName + " where bm = ? and xm = ? and ny = ? and jblb = ? ";
            log.info("-WorkOverTimeAction- sql2：{}",sql2);
            rs.executeQuery(sql2, bm,xm,fromdate,field28);
            if (rs.next()) {
                id = rs.getString("id");
            }
            log.info("-WorkOverTimeAction- id：{}",id);
            Map<String, String> tableNameMap = getDtlTableName(changetype, mainTableName);
            String dtlTableName = tableNameMap.get("dtlTableName");
            String updateName = tableNameMap.get("updateName");
            log.info("-WorkOverTimeAction- tableNameMap：{}",tableNameMap);
            String insert_detail;
            String updateSql;
            String conOvertimeModeId = "";
            if (StringUtils.isBlank(id)) {
                // 2. 新增主表信息，再新增明细表信息
                String insert_main = "insert into " + mainTableName + " (bm,xm,ny,jblb,formmodeid, modedatacreater, modedatacreatertype, modedatacreatedate, modedatacreatetime, MODEUUID) " +
                        " values (?,?,?,?,?,?,?,?,?,?) ";
                // 3.权限重构
                if(StringUtils.equals(mainTableName,"uf_jbtxdyfjs")){
                    //当月
                    conOvertimeModeId = bb.getPropValue("project_sskj", "workOvertimeDyModeId");
                }else{
                    //隔月
                    conOvertimeModeId = bb.getPropValue("project_sskj", "workOvertimeGyModeId");
                }
                Integer modedatacreater = 1;
                Integer modedatacreatertype = 0;
                String modedatacreatedate = DateUtil.format(new Date(), "yyyy-MM-dd");
                String modedatacreatetime = DateUtil.format(new Date(), "HH:mm:ss");
                String uuid = UUID.randomUUID().toString();
                log.info("-WorkOverTimeAction-insert_main sql：[{}]",insert_main);
                boolean b = rs.executeUpdate(insert_main, bm, xm, fromdate,field28, conOvertimeModeId, modedatacreater, modedatacreatertype, modedatacreatedate, modedatacreatetime, uuid);
                log.info("addFlag：{}" , b);
                if (b) {
                    String billid = "-1";
                    // 2. 先查询主表id
                    //String main_id = "";
                    //String sql3 = " select id from " + mainTableName + " where bm = ? and xm = ? and ny = ? and jblb = ? ";
                    String sql3 = " select id from " + mainTableName + " where MODEUUID = ? ";
                    log.info("-WorkOverTimeAction- sql3：{}",sql3);
                    rs.executeQuery(sql3, uuid);
                    if (rs.next()) {
                        billid = Util.null2String(rs.getString("id"));
                    }
                    log.info("billid：{}" , billid);
                    sskjUtil.modePerRecon(modedatacreater, conOvertimeModeId, billid);
                    // 3.新增明细表数据,需先判断加班类型
                     insert_detail = "insert into "+ dtlTableName + " (mainid,jbry,szbm,gw,ksrq,kssj,jsrq,jssj,jbsch,xglc,lcgzrq) values (?,?,?,?,?,?,?,?,?,?,?) ";
                     rs.executeUpdate(insert_detail, billid,xm,bm,map.get("jobtitle"), map.get("fromdate"),
                             map.get("fromtime"), map.get("todate"),map.get("totime"),map.get("duration"),requestId,map.get("belongdate"));
                     // 4.更新主表信息
                    updateSql = "update " + mainTableName + " set " + updateName + "= ?,jbzsy = ? where id = ? ";
                    log.info("更新首条加班时长到主表-sql:{}",updateSql);
                    rs.executeUpdate(updateSql,map.get("duration"),map.get("duration"),billid);
                }else{
                    log.info("加班申请流程转建模数据失败！");
                    return Action.FAILURE_AND_CONTINUE;
                }
            } else {
                // 更新主表信息之前，先查询有没有事假记录
                boolean b1 = sskjUtil.handleCasLeave(xm, map.get("belongdate"), requestId, Double.parseDouble(map.get("duration")));
                log.info("处理有事假情况结果-b1:{}",b1);
                if(!b1){
                    log.info("加班申请流程转建模数据失败！");
                    return Action.FAILURE_AND_CONTINUE;
                }
                // 1.直接增加明细表数据
                insert_detail = "insert into "+ dtlTableName + " (mainid,jbry,szbm,gw,ksrq,kssj,jsrq,jssj,jbsch,xglc,lcgzrq) values (?,?,?,?,?,?,?,?,?,?,?)";
                boolean b = rs.executeUpdate(insert_detail, id, xm, bm, map.get("jobtitle"), map.get("fromdate"),
                        map.get("fromtime"), map.get("todate"), map.get("totime"), map.get("duration"),requestId,map.get("belongdate"));
                if(!b){
                    log.info("加班申请流程转建模数据失败！");
                    return Action.FAILURE_AND_CONTINUE;
                }
                // 明细表加班时长求和
                //String value = "";
                String sum = "";
                String sql = "select sum(jbsch) as num from  "+ dtlTableName + " b WHERE b.mainid = ? ";
                rs.executeQuery(sql, id);
                if (rs.next()) {
                    //value = rs.getString(updateName);
                    sum = rs.getString("num");
                }
                double updateValue = Double.parseDouble(sum);
                log.info("加班累计时长-updateValue:{}",updateValue);
                // 更新最终求和到主表待更新字段
                updateSql =  "update " + mainTableName + " set " + updateName + " = ? where id = ? ";
                log.info("更新累计加班时长到主表-sql:{}",updateSql);
                rs.executeUpdate(updateSql,updateValue,id);
                String balance = getOvertimeBalance(mainTableName, bm, xm, map.get("fromdate"), field28);
                log.info("加班总剩余-balance:{}",balance);
                // 更新最终求和到主表待更新字段
                updateSql =  "update " + mainTableName + " set jbzsy = ? where id = ? ";
                log.info("更新累计加班时长到主表-sql:{}",updateSql);
                rs.executeUpdate(updateSql,balance,id);

            }
            log.info("-WorkOverTimeAction-insert_detail sql：[{}]",insert_detail);

     }
        log.info("加班申请流程转建模数据成功！");
        return Action.SUCCESS;
    }

    /**
     * 获取当前月加班余额
     *
     * @return
     */
    public static String getOvertimeBalance(String tableName, String bm, String xm, String fromDate,String field28) {
        RecordSet rs = new RecordSet();
        String jbzsy = "";
        String sql = " select IF(IF(psjbsy is null,psjb,psjbsy) is null,0,IF(psjbsy is null,psjb,psjbsy)) + " +
                " IF(IF(zmjbsy is null,zmjb,zmjbsy) is null,0,IF(zmjbsy is null,zmjb,zmjbsy)) + " +
                " IF(IF(jrjbsy is null,jrjb,jrjbsy) is null,0,IF(jrjbsy is null,jrjb,jrjbsy)) as jbzsy  from " + tableName + " where bm = ? and xm = ? and ny = ? and jblb = ?";
        log.info("-getOvertimeBalance- sql：{}", sql);
        rs.executeQuery(sql, bm, xm, fromDate.substring(0, 7),field28);
        if (rs.next()) {
            jbzsy = rs.getString("jbzsy");
        }
        log.info("-getOvertimeBalance- jbzsy：{}", jbzsy);
        return jbzsy;
    }

    /**
     * 根据field28获取建模主表名称
     * */
    public static String getMainTableName(String field28){
        String mainTableName = "";
        if(StringUtils.equals(field28,"4")){//当月发放
            mainTableName = "uf_jbtxdyfjs";
        }else if(StringUtils.equals(field28,"5")||StringUtils.equals(field28,"6")){//隔月发放/三月发放
            mainTableName = "uf_jbtxgyfff";
        }else{
            //默认值(暂时)
            mainTableName = "uf_jbtxdyfjs";
        }
        return mainTableName;
    }

    /**
     * 根据changetype获取建模明细表名称
     * */
    public static Map<String,String> getDtlTableName(String changetype,String name){
        Map<String,String> map = new HashMap<>();
        //1-节假日、2-工作日、3-休息日
        String dtlTableName = "";
        String updateName = "";
        if(StringUtils.equals(changetype,"1")){
            dtlTableName = name+"_dt1";
            updateName = "jrjb";
        }else if(StringUtils.equals(changetype,"2")){
            dtlTableName = name+"_dt3";
            updateName = "psjb";
        }else if(StringUtils.equals(changetype,"3")){
            dtlTableName = name+"_dt4";
            updateName = "zmjb";
        }
        map.put("dtlTableName",dtlTableName);
        map.put("updateName",updateName);
        log.info("获取明细表信息-map{}",map);
        return map;
    }

    /**
     * 根据requestInfo获取考勤表数据
     * */
    public static List<Map<String,String>> getKqFlowOverTimeInfo(String requestId){
        List<Map<String,String>> list = new ArrayList<>();
        RecordSet rs = new RecordSet();
        String resourceid;
        String fromdate;
        String fromtime;
        String todate;
        String totime;
        String belongdate;
        String duration;
        String changetype;
        String departmentid;
        String jobtitle;
        String field28;

        String sql = "select resourceid,fromdate,fromtime,todate,totime,belongdate,duration,changetype,departmentid,jobtitle,field28 from kq_flow_split_overtime a left join " +
                "cus_fielddata b on a.resourceid = b.id   where SCOPE = 'HrmCustomFieldByInfoType' and SCOPEID = '3' and a.requestId = ?";
        rs.executeQuery(sql,requestId);
        while (rs.next()){
            Map<String,String> map = new HashMap<>();
            resourceid = rs.getString("resourceid");
            fromdate = rs.getString("fromdate");
            fromtime = rs.getString("fromtime");
            todate = rs.getString("todate");
            totime = rs.getString("totime");
            belongdate = rs.getString("belongdate");
            duration = rs.getString("duration");
            changetype = rs.getString("changetype");
            departmentid = rs.getString("departmentid");
            jobtitle = rs.getString("jobtitle");
            field28 = rs.getString("field28");
            map.put("resourceid",resourceid);
            map.put("fromdate",fromdate);
            map.put("fromtime",fromtime);
            map.put("todate",todate);
            map.put("totime",totime);
            map.put("belongdate",belongdate);
            map.put("duration",duration);
            map.put("changetype",changetype);
            map.put("departmentid",departmentid);
            map.put("jobtitle",jobtitle);
            map.put("field28",field28);
            list.add(map);
        }
        log.info("getKqFlowOverTimeInfo-list:{}",list);
        return list;
    }

    /**
     * 根据requestInfo获取主表数据
     * */
    public static Map<String,Object> getMainInfo(RequestInfo requestInfo){
        Map<String,Object> map = new HashMap();
        Property[] property = requestInfo.getMainTableInfo().getProperty();
        for (int i = 0; i < property.length; i++) {
            map.put(property[i].getName().toLowerCase(), Util.null2String(property[i].getValue()));
        }
        return map;
    }

    /**
     * 根据requestInfo获取明细表数据
     * */
    public static List getDetailInfo(RequestInfo requestInfo, int num){
        DetailTable detailTable = requestInfo.getDetailTableInfo().getDetailTable(num);
        //dtltable数组中的行数据集合
        Row[] rows = detailTable.getRow();
        List sublist = new ArrayList();
        for (int i = 0; i < rows.length; i++) {
            Row row = rows[i];
            Map onerow = new HashMap();
            sublist.add(onerow);
            Cell[] cells = row.getCell();
            for (int j = 0; j < cells.length; j++) {
                Cell cell = cells[j];
                onerow.put(cell.getName(), Util.null2String(cell.getValue()));
            }
        }
        return sublist;
    }


}
